@@ -1,4 +1,6 @@
-"""Methods for advection-based extrapolation of precipitation fields.
+# -*- coding: utf-8 -*-
+"""
+    Methods for advection-based extrapolation of precipitation fields.
 Currently the module contains an implementation of the
 semi-Lagrangian method described in :cite:`GZ2002` and the
 eulerian persistence."""
