@@ -48,7 +48,8 @@ def forecast(
     filter_kwargs=None,
     measure_time=False,
 ):
-    """Generate a nowcast by using the Spectral Prognosis (S-PROG) method.
+    """
+    Generate a nowcast by using the Spectral Prognosis (S-PROG) method.
 
     Parameters
     ----------
@@ -461,7 +462,7 @@ def _compute_sprog_mask(R, war):
     i = np.argmin(abs(x - war))
     # handle ties
     if R_s[i] == R_s[i + 1]:
-        i = np.where(R_s == R_s[i])[0][-1] + 1
+        i = np.where(R_s == R_s[i])[0][-1]
     R_pct_thr = R_s[i]
 
     # determine a mask using the above threshold value to preserve the
