@@ -1,3 +1,4 @@
+# -- coding: utf-8 --
 """
 pysteps.visualization.precipfields
 ==================================
@@ -10,27 +11,28 @@ Methods for plotting precipitation fields.
     plot_precip_field
     get_colormap
 """
+import copy
+import warnings
 
 import matplotlib.pylab as plt
-from matplotlib import cm, colors  # , gridspec
 import numpy as np
+from matplotlib import cm, colors
 
-try:
-    import pyproj
+from pysteps.visualization.utils import get_geogrid, get_basemap_axis
 
-    PYPROJ_IMPORTED = True
-except ImportError:
-    PYPROJ_IMPORTED = False
-from pysteps.exceptions import MissingOptionalDependency, UnsupportedSomercProjection
+PRECIP_VALID_TYPES = ("intensity", "depth", "prob")
+PRECIP_VALID_UNITS = ("mm/h", "mm", "dBZ")
 
-from . import basemaps
-from . import utils
+
+############################
+# precipitation plots zorder
+# - precipitation: 10
 
 
 def plot_precip_field(
-    R,
-    type="intensity",
-    map=None,
+    precip,
+    ptype="intensity",
+    ax=None,
     geodata=None,
     units="mm/h",
     bbox=None,
@@ -38,11 +40,9 @@ def plot_precip_field(
     probthr=None,
     title=None,
     colorbar=True,
-    drawlonlatlines=False,
-    lw=0.5,
     axis="on",
     cax=None,
-    **kwargs,
+    map_kwargs=None,
 ):
     """
     Function to plot a precipitation intensity or probability field with a
@@ -52,23 +52,16 @@ def plot_precip_field(
 
     .. _SubplotSpec: https://matplotlib.org/api/_as_gen/matplotlib.gridspec.SubplotSpec.html
 
-    .. _cartopy: https://scitools.org.uk/cartopy/docs/latest
-
-    .. _mpl_toolkits.basemap: https://matplotlib.org/basemap
-
     Parameters
     ----------
-    R : array-like
+    precip: array-like
         Two-dimensional array containing the input precipitation field or an
         exceedance probability map.
-    type : {'intensity', 'depth', 'prob'}, optional
+    ptype: {'intensity', 'depth', 'prob'}, optional
         Type of the map to plot: 'intensity' = precipitation intensity field,
         'depth' = precipitation depth (accumulation) field,
         'prob' = exceedance probability field.
-    map : {'basemap', 'cartopy'}, optional
-        Optional method for plotting a map: 'basemap' or 'cartopy'. The former
-        uses `mpl_toolkits.basemap`_, while the latter uses cartopy_.
-    geodata : dictionary, optional
+    geodata: dictionary or None, optional
         Optional dictionary containing geographical information about
         the field. Required is map is not None.
 
@@ -98,29 +91,24 @@ def plot_precip_field(
         |                 | 'upper' = upper border, 'lower' = lower border    |
         +-----------------+---------------------------------------------------+
     units : {'mm/h', 'mm', 'dBZ'}, optional
-        Units of the input array. If type is 'prob', this specifies the unit of
+        Units of the input array. If ptype is 'prob', this specifies the unit of
         the intensity threshold.
     bbox : tuple, optional
         Four-element tuple specifying the coordinates of the bounding box. Use
         this for plotting a subdomain inside the input grid. The coordinates are
-        of the form (lower left x,lower left y,upper right x,upper right y). If
-        map is not None, the x- and y-coordinates are longitudes and latitudes.
-        Otherwise they represent image pixels.
+        of the form (lower left x, lower left y ,upper right x, upper right y).
+        If 'geodata' is not None, the bbox is in map coordinates, otherwise
+        it represents image pixels.
     colorscale : {'pysteps', 'STEPS-BE', 'BOM-RF3'}, optional
         Which colorscale to use. Applicable if units is 'mm/h', 'mm' or 'dBZ'.
     probthr : float, optional
         Intensity threshold to show in the color bar of the exceedance
         probability map.
-        Required if type is "prob" and colorbar is True.
+        Required if ptype is "prob" and colorbar is True.
     title : str, optional
         If not None, print the title on top of the plot.
     colorbar : bool, optional
         If set to True, add a colorbar on the right side of the plot.
-    drawlonlatlines : bool, optional
-        If set to True, draw longitude and latitude lines. Applicable if map is
-        'basemap' or 'cartopy'.
-    lw: float, optional
-        Linewidth of the map (administrative boundaries and coastlines).
     axis : {'off','on'}, optional
         Whether to turn off or on the x and y axis.
     cax : Axes_ object, optional
@@ -129,248 +117,134 @@ def plot_precip_field(
 
     Other parameters
     ----------------
-    Optional parameters are contained in **kwargs. See basemaps.plot_geography.
+    map_kwargs: dict
+        Optional parameters that need to be passed to
+        :py:func:`pysteps.visualization.basemaps.plot_geography`.
 
     Returns
     -------
     ax : fig Axes_
         Figure axes. Needed if one wants to add e.g. text inside the plot.
-
     """
-    if map is not None:
-        FutureWarning(
-            "'map' argument will be renamed to 'plot_map' in 1.4.0. Use 'plot_map' to silence this warning."
-        )
-        plot_map = map
-    else:
-        plot_map = kwargs.pop("plot_map", None)
 
-    if type not in ["intensity", "depth", "prob"]:
+    if map_kwargs is None:
+        map_kwargs = {}
+
+    if ptype not in PRECIP_VALID_TYPES:
         raise ValueError(
-            "invalid type '%s', must be " + "'intensity', 'depth' or 'prob'" % type
+            f"Invalid precipitation type '{ptype}'."
+            f"Supported: {str(PRECIP_VALID_TYPES)}"
         )
-    if units not in ["mm/h", "mm", "dBZ"]:
+
+    if units not in PRECIP_VALID_UNITS:
         raise ValueError(
-            "invalid units '%s', must be " + "'mm/h', 'mm' or 'dBZ'" % units
+            f"Invalid precipitation units '{units}."
+            f"Supported: {str(PRECIP_VALID_UNITS)}"
         )
-    if type == "prob" and colorbar and probthr is None:
-        raise ValueError("type='prob' but probthr not specified")
-    if plot_map is not None and geodata is None:
-        raise ValueError("map!=None but geodata=None")
-    if len(R.shape) != 2:
-        raise ValueError("the input is not two-dimensional array")
 
-    # get colormap and color levels
-    cmap, norm, clevs, clevsStr = get_colormap(type, units, colorscale)
+    if ptype == "prob" and colorbar and probthr is None:
+        raise ValueError("ptype='prob' but probthr not specified")
 
-    # extract extent and origin
-    if geodata is not None:
-        field_extent = (geodata["x1"], geodata["x2"], geodata["y1"], geodata["y2"])
-        if bbox is None:
-            bm_extent = field_extent
-        else:
-            if not PYPROJ_IMPORTED:
-                raise MissingOptionalDependency(
-                    "pyproj package is required to import "
-                    "FMI's radar reflectivity composite "
-                    "but it is not installed"
-                )
-            pr = pyproj.Proj(geodata["projection"])
-            x1, y1 = pr(bbox[0], bbox[1])
-            x2, y2 = pr(bbox[2], bbox[3])
-            bm_extent = (x1, x2, y1, y2)
-        origin = geodata["yorigin"]
-    else:
-        field_extent = (0, R.shape[1] - 1, 0, R.shape[0] - 1)
-        origin = "upper"
+    if len(precip.shape) != 2:
+        raise ValueError("The input is not two-dimensional array")
 
-    # plot geography
-    if plot_map is not None:
-        try:
-            ax = basemaps.plot_geography(
-                plot_map,
-                geodata["projection"],
-                bm_extent,
-                lw=lw,
-                drawlonlatlines=drawlonlatlines,
-                **kwargs,
-            )
-            regular_grid = True
-        except UnsupportedSomercProjection:
-            # Define default fall-back projection for Swiss data(EPSG:3035)
-            # This will work reasonably well for Europe only.
-            t_proj4str = "+proj=laea +lat_0=52 +lon_0=10 "
-            t_proj4str += "+x_0=4321000 +y_0=3210000 +ellps=GRS80 "
-            t_proj4str += "+units=m +no_defs"
-            geodata = utils.reproject_geodata(
-                geodata, t_proj4str, return_grid="quadmesh"
-            )
-            bm_extent = (geodata["x1"], geodata["x2"], geodata["y1"], geodata["y2"])
-            X, Y = geodata["X_grid"], geodata["Y_grid"]
-            regular_grid = geodata["regular_grid"]
+    # Assumes the input dimensions are lat/lon
+    nlat, nlon = precip.shape
 
-            ax = basemaps.plot_geography(
-                plot_map,
-                geodata["projection"],
-                bm_extent,
-                lw=lw,
-                drawlonlatlines=drawlonlatlines,
-                **kwargs,
-            )
-    else:
-        regular_grid = True
-
-    if bbox is not None and plot_map is not None:
-        x1, y1 = pr(geodata["x1"], geodata["y1"], inverse=True)
-        x2, y2 = pr(geodata["x2"], geodata["y2"], inverse=True)
-        if plot_map == "basemap":
-            x1, y1 = ax(x1, y1)
-            x2, y2 = ax(x2, y2)
-        else:
-            x1, y1 = pr(x1, y1)
-            x2, y2 = pr(x2, y2)
-        field_extent = (x1, x2, y1, y2)
-
-    # plot rainfield
-    if regular_grid:
-        ax = plt.gca()
-        im = _plot_field(
-            R, ax, type, units, colorscale, extent=field_extent, origin=origin
-        )
-    else:
-        if origin == "upper":
-            Y = np.flipud(Y)
-        im = _plot_field_pcolormesh(X, Y, R, ax, type, units, colorscale)
-
-    # plot radar domain mask
-    mask = np.ones(R.shape)
-    mask[~np.isnan(R)] = np.nan  # Fully transparent within the radar domain
-    ax.imshow(
-        mask,
-        cmap=colors.ListedColormap(["gray"]),
-        alpha=0.5,
-        zorder=1e6,
-        extent=field_extent,
-        origin=origin,
+    x_grid, y_grid, extent, regular_grid, origin = get_geogrid(
+        nlat, nlon, geodata=geodata
     )
 
-    # ax.pcolormesh(X, Y, np.flipud(mask),
-    #               cmap=colors.ListedColormap(['gray']),
-    #               alpha=0.5, zorder=1e6)
-    # TODO: pcolormesh doesn't work properly with the alpha parameter
+    ax = get_basemap_axis(extent, ax=ax, geodata=geodata, map_kwargs=map_kwargs)
 
-    if title is not None:
-        plt.title(title)
+    precip = np.ma.masked_invalid(precip)
+    # plot rainfield
+    if regular_grid:
+        im = _plot_field(precip, ax, ptype, units, colorscale, extent, origin=origin)
+    else:
+        im = _plot_field(
+            precip, ax, ptype, units, colorscale, extent, x_grid=x_grid, y_grid=y_grid
+        )
+
+    plt.title(title)
 
     # add colorbar
     if colorbar:
-        if type in ["intensity", "depth"]:
+        # get colormap and color levels
+        _, _, clevs, clevs_str = get_colormap(ptype, units, colorscale)
+        if ptype in ["intensity", "depth"]:
             extend = "max"
         else:
             extend = "neither"
         cbar = plt.colorbar(
-            im,
-            ticks=clevs,
-            spacing="uniform",
-            norm=norm,
-            extend=extend,
-            shrink=0.8,
-            cax=cax,
+            im, ticks=clevs, spacing="uniform", extend=extend, shrink=0.8, cax=cax
         )
-        if clevsStr is not None:
-            cbar.ax.set_yticklabels(clevsStr)
+        if clevs_str is not None:
+            cbar.ax.set_yticklabels(clevs_str)
 
-        if type == "intensity":
-            cbar.ax.set_title(units, fontsize=10)
-            cbar.set_label("Precipitation intensity")
-        elif type == "depth":
-            cbar.ax.set_title(units, fontsize=10)
-            cbar.set_label("Precipitation depth")
+        if ptype == "intensity":
+            cbar.set_label(f"Precipitation intensity [{units}]")
+        elif ptype == "depth":
+            cbar.set_label(f"Precipitation depth [{units}]")
         else:
-            cbar.set_label("P(R > %.1f %s)" % (probthr, units))
+            cbar.set_label(f"P(R > {probthr:.1f} {units})")
 
-    if plot_map is None and bbox is not None:
-        ax = plt.gca()
+    if geodata is None or axis == "off":
+        ax.xaxis.set_ticks([])
+        ax.xaxis.set_ticklabels([])
+        ax.yaxis.set_ticks([])
+        ax.yaxis.set_ticklabels([])
+
+    if bbox is not None:
         ax.set_xlim(bbox[0], bbox[2])
         ax.set_ylim(bbox[1], bbox[3])
 
-    if geodata is None or axis == "off":
-        axes = plt.gca()
-        axes.xaxis.set_ticks([])
-        axes.xaxis.set_ticklabels([])
-        axes.yaxis.set_ticks([])
-        axes.yaxis.set_ticklabels([])
-
-    return plt.gca()
+    return ax
 
 
-def _plot_field(R, ax, type, units, colorscale, extent, origin=None):
-    R = R.copy()
+def _plot_field(
+    precip, ax, ptype, units, colorscale, extent, origin=None, x_grid=None, y_grid=None
+):
+    precip = precip.copy()
 
     # Get colormap and color levels
-    cmap, norm, clevs, clevsStr = get_colormap(type, units, colorscale)
+    cmap, norm, _, _ = get_colormap(ptype, units, colorscale)
 
-    # Plot precipitation field
-    # transparent where no precipitation or the probability is zero
-    if type in ["intensity", "depth"]:
-        if units in ["mm/h", "mm"]:
-            R[R < 0.1] = np.nan
-        elif units == "dBZ":
-            R[R < 10] = np.nan
+    if (x_grid is None) or (y_grid is None):
+        im = ax.imshow(
+            precip,
+            cmap=cmap,
+            norm=norm,
+            extent=extent,
+            interpolation="nearest",
+            origin=origin,
+            zorder=10,
+        )
     else:
-        R[R < 1e-3] = np.nan
-
-    vmin, vmax = [None, None] if type in ["intensity", "depth"] else [0.0, 1.0]
-
-    im = ax.imshow(
-        R,
-        cmap=cmap,
-        norm=norm,
-        extent=extent,
-        interpolation="nearest",
-        vmin=vmin,
-        vmax=vmax,
-        origin=origin,
-        zorder=1,
-    )
+        im = ax.pcolormesh(
+            x_grid,
+            y_grid,
+            precip,
+            cmap=cmap,
+            norm=norm,
+            zorder=10,
+        )
 
     return im
 
 
-def _plot_field_pcolormesh(X, Y, R, ax, type, units, colorscale):
-    R = R.copy()
-
-    # Get colormap and color levels
-    cmap, norm, clevs, clevsStr = get_colormap(type, units, colorscale)
-
-    # Plot precipitation field
-    # transparent where no precipitation or the probability is zero
-    if type in ["intensity", "depth"]:
-        if units in ["mm/h", "mm"]:
-            R[R < 0.1] = np.nan
-        elif units == "dBZ":
-            R[R < 10] = np.nan
-    else:
-        R[R < 1e-3] = np.nan
-
-    vmin, vmax = [None, None] if type in ["intensity", "depth"] else [0.0, 1.0]
-
-    im = ax.pcolormesh(X, Y, R, cmap=cmap, norm=norm, vmin=vmin, vmax=vmax, zorder=1)
-
-    return im
-
-
-def get_colormap(type, units="mm/h", colorscale="pysteps"):
-    """Function to generate a colormap (cmap) and norm.
+def get_colormap(ptype, units="mm/h", colorscale="pysteps"):
+    """
+    Function to generate a colormap (cmap) and norm.
 
     Parameters
     ----------
-    type : {'intensity', 'depth', 'prob'}, optional
+    ptype : {'intensity', 'depth', 'prob'}, optional
         Type of the map to plot: 'intensity' = precipitation intensity field,
         'depth' = precipitation depth (accumulation) field,
         'prob' = exceedance probability field.
     units : {'mm/h', 'mm', 'dBZ'}, optional
-        Units of the input array. If type is 'prob', this specifies the unit of
+        Units of the input array. If ptype is 'prob', this specifies the unit of
         the intensity threshold.
     colorscale : {'pysteps', 'STEPS-BE', 'BOM-RF3'}, optional
         Which colorscale to use. Applicable if units is 'mm/h', 'mm' or 'dBZ'.
@@ -383,14 +257,13 @@ def get_colormap(type, units="mm/h", colorscale="pysteps"):
         Colors norm
     clevs: list(float)
         List of precipitation values defining the color limits.
-    clevsStr: list(str)
+    clevs_str: list(str)
         List of precipitation values defining the color limits (with correct
         number of decimals).
-
     """
-    if type in ["intensity", "depth"]:
+    if ptype in ["intensity", "depth"]:
         # Get list of colors
-        color_list, clevs, clevsStr = _get_colorlist(units, colorscale)
+        color_list, clevs, clevs_str = _get_colorlist(units, colorscale)
 
         cmap = colors.LinearSegmentedColormap.from_list(
             "cmap", color_list, len(clevs) - 1
@@ -404,13 +277,22 @@ def get_colormap(type, units="mm/h", colorscale="pysteps"):
             cmap.set_over("black", 1)
         norm = colors.BoundaryNorm(clevs, cmap.N)
 
-        return cmap, norm, clevs, clevsStr
+        cmap.set_bad("gray", alpha=0.5)
+        cmap.set_under("none")
 
-    elif type == "prob":
-        cmap = plt.get_cmap("OrRd", 10)
-        return cmap, colors.Normalize(vmin=0, vmax=1), None, None
-    else:
-        return cm.jet, colors.Normalize(), None, None
+        return cmap, norm, clevs, clevs_str
+
+    if ptype == "prob":
+        cmap = copy.copy(plt.get_cmap("OrRd", 10))
+        cmap.set_bad("gray", alpha=0.5)
+        cmap.set_under("none")
+        clevs = np.linspace(0, 1, 11)
+        clevs[0] = 1e-3  # to set zeros to transparent
+        norm = colors.BoundaryNorm(clevs, cmap.N)
+        clevs_str = [f"{clev:.1f}" for clev in clevs]
+        return cmap, norm, clevs, clevs_str
+
+    return cm.get_cmap("jet"), colors.Normalize(), None, None
 
 
 def _get_colorlist(units="mm/h", colorscale="pysteps"):
@@ -432,10 +314,9 @@ def _get_colorlist(units="mm/h", colorscale="pysteps"):
     clevs : list(float)
         List of precipitation values defining the color limits.
 
-    clevsStr : list(str)
+    clevs_str : list(str)
         List of precipitation values defining the color limits
         (with correct number of decimals).
-
     """
 
     if colorscale == "BOM-RF3":
@@ -504,9 +385,9 @@ def _get_colorlist(units="mm/h", colorscale="pysteps"):
             raise ValueError("Wrong units in get_colorlist: %s" % units)
     elif colorscale == "pysteps":
         # pinkHex = '#%02x%02x%02x' % (232, 215, 242)
-        redgreyHex = "#%02x%02x%02x" % (156, 126, 148)
+        redgrey_hex = "#%02x%02x%02x" % (156, 126, 148)
         color_list = [
-            redgreyHex,
+            redgrey_hex,
             "#640064",
             "#AF00AF",
             "#DC00DC",
@@ -576,30 +457,27 @@ def _get_colorlist(units="mm/h", colorscale="pysteps"):
         raise ValueError("Invalid colorscale " + colorscale)
 
     # Generate color level strings with correct amount of decimal places
-    clevsStr = []
-    clevsStr = _dynamic_formatting_floats(clevs,)
+    clevs_str = _dynamic_formatting_floats(clevs)
 
-    return color_list, clevs, clevsStr
+    return color_list, clevs, clevs_str
 
 
-def _dynamic_formatting_floats(floatArray, colorscale="pysteps"):
-    """
-    Function to format the floats defining the class limits of the colorbar.
-    """
-    floatArray = np.array(floatArray, dtype=float)
+def _dynamic_formatting_floats(float_array, colorscale="pysteps"):
+    """Function to format the floats defining the class limits of the colorbar."""
+    float_array = np.array(float_array, dtype=float)
 
     labels = []
-    for label in floatArray:
-        if label >= 0.1 and label < 1:
+    for label in float_array:
+        if 0.1 <= label < 1:
             if colorscale == "pysteps":
                 formatting = ",.2f"
             else:
                 formatting = ",.1f"
-        elif label >= 0.01 and label < 0.1:
+        elif 0.01 <= label < 0.1:
             formatting = ",.2f"
-        elif label >= 0.001 and label < 0.01:
+        elif 0.001 <= label < 0.01:
             formatting = ",.3f"
-        elif label >= 0.0001 and label < 0.001:
+        elif 0.0001 <= label < 0.001:
             formatting = ",.4f"
         elif label >= 1 and label.is_integer():
             formatting = "i"
